/***********************************************************\
Copyright (C) James Boer, 2002. 
All rights reserved worldwide.

This software is provided "as is" without express or implied
warranties. You may freely copy and compile this source into
applications you distribute provided that the copyright text
below is included in the resulting source code, for example:
"Portions Copyright (C) James Boer, 2002"
\***********************************************************/

#include "stdafx.h"
#include "audiotest.h"
#include "MusicScript.h"

#include "AudioTest.h"
#include "Segment.h"
#include "SoundOpen.h"
#include "Audio.h"

#include "AudioTestDlg.h"
#include "System.h"
#include "Sound2D.h"
#include "Sound3D.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

using namespace Audio;

#define GET_CURRENT_THEME() \
	CString sTheme; \
	if(m_ThemeList.GetText(m_ThemeList.GetCurSel(), sTheme) == LB_ERR ) return;

/////////////////////////////////////////////////////////////////////////////
// CMusicScript property page

IMPLEMENT_DYNCREATE(CMusicScript, CPropertyPage)

CMusicScript::CMusicScript() : CPropertyPage(CMusicScript::IDD)
{
	//{{AFX_DATA_INIT(CMusicScript)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	m_bInit = false;
}

CMusicScript::~CMusicScript()
{
}

void CMusicScript::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMusicScript)
	DDX_Control(pDX, IDC_LIST_MS_SOUNDS, m_ThemeList);
	DDX_Control(pDX, IDC_STOPMS, m_Stop);
	DDX_Control(pDX, IDC_PLAYMS, m_Play);
	DDX_Control(pDX, IDC_PAUSEMS, m_Pause);
	//}}AFX_DATA_MAP
}

void CMusicScript::ClearList()
{
	if(m_bInit)
	{
		MusicMgr()->RemoveAll();
		m_ThemeList.ResetContent();
	}
}


void CMusicScript::UpdateControls()
{
}



BEGIN_MESSAGE_MAP(CMusicScript, CPropertyPage)
	//{{AFX_MSG_MAP(CMusicScript)
	ON_WM_DESTROY()
	ON_BN_CLICKED(IDC_MS_INITIALIZE, OnMSInitialize)
	ON_BN_CLICKED(IDC_PLAYMS, OnPlayMS)
	ON_BN_CLICKED(IDC_PAUSEMS, OnPauseMS)
	ON_BN_CLICKED(IDC_STOPMS, OnStopMS)
	ON_BN_CLICKED(IDC_UNLOADMS, OnUnloadMS)
	ON_LBN_DBLCLK(IDC_LIST_MS_SOUNDS, OnDblclkListMsSounds)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMusicScript message handlers

BOOL CMusicScript::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();
	
	// Load the play button icons
	m_hPlayIcon = LoadIcon(::AfxGetInstanceHandle(), MAKEINTRESOURCE(IDI_PLAY));
	m_hPauseIcon = LoadIcon(::AfxGetInstanceHandle(), MAKEINTRESOURCE(IDI_PAUSE));
	m_hStopIcon = LoadIcon(::AfxGetInstanceHandle(), MAKEINTRESOURCE(IDI_STOP));

	// Set the master play control buttons
	m_Play.SetIcon(m_hPlayIcon);
	m_Pause.SetIcon(m_hPauseIcon);
	m_Stop.SetIcon(m_hStopIcon);

	m_bInit = true;
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CMusicScript::OnDestroy() 
{
	CPropertyPage::OnDestroy();
	ClearList();
}

void CMusicScript::OnMSInitialize() 
{
	CFileDialog dlg(TRUE, NULL, NULL, OFN_ALLOWMULTISELECT | OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
		"Script Files (.txt)|*.txt|All Files (*.*)|*.*||");

	if(dlg.DoModal() != IDOK)
		return;


	char drive[_MAX_DRIVE];
	char dir[_MAX_DIR];
	char fname[_MAX_FNAME];
	char ext[_MAX_EXT];

	POSITION pos = dlg.GetStartPosition();
	CString sPathName;
	while(pos)
	{
		sPathName = dlg.GetNextPathName(pos);
		_splitpath(sPathName, drive, dir, fname, ext);

		if(!MusicMgr()->LoadScript((const char*)sPathName))
		{
			AfxMessageBox("Error parsing sound script");
			return;
		}

	}

	m_ThemeList.ResetContent();
	std::string sName;
	MusicMgr()->ResetThemeItr();
	while(MusicMgr()->GetNextTheme(sName))
	{
		DebugOut(1, "Registered soundscape = %s", sName.c_str());
		m_ThemeList.AddString(sName.c_str());
	}
}

void CMusicScript::OnUnloadMS() 
{
	ClearList();
}

void CMusicScript::OnPlayMS() 
{
	CString sTheme;
	int iIndex = m_ThemeList.GetCurSel();
	if(iIndex == LB_ERR)
		return;
	m_ThemeList.GetText(iIndex, sTheme);
	
	MusicMgr()->PlayTheme((const char*)sTheme);
}

void CMusicScript::OnPauseMS() 
{
	ISegment* pSegment;
	if(AudioMgr()->GetCurrentSegment(pSegment))
		pSegment->Pause();	
}

void CMusicScript::OnStopMS() 
{
	ISegment* pSegment;
	if(AudioMgr()->GetCurrentSegment(pSegment))
		pSegment->Stop();	
}

void CMusicScript::OnDblclkListMsSounds() 
{
	OnPlayMS();	
}
